@@ -23,7 +23,7 @@ from ..._ortho_learner import _OrthoLearner
 from ..._cate_estimator import LinearModelFinalCateEstimatorMixin, StatsModelsCateEstimatorMixin
 from ...inference import StatsModelsInference
 from ...sklearn_extensions.linear_model import StatsModelsLinearRegression
-from ...utilities import _deprecate_positional
+from ...utilities import _deprecate_positional, get_feature_names_or_default
 from .._nuisance_wrappers import _FirstStageWrapper, _FinalWrapper
 
 
@@ -42,7 +42,7 @@ class _BaseDMLATEIVModelFinal:
                                            sample_weight=sample_weight).predict(Z_res)
         # TODO: allow the final model to actually use X? Then we'd need to rename the class
         #       since we would actually be calculating a CATE rather than ATE.
-        self._model_final.fit(X=None, T_res=T_res_pred, Y_res=Y_res, sample_weight=sample_weight)
+        self._model_final.fit(X=None, T=T, T_res=T_res_pred, Y_res=Y_res, sample_weight=sample_weight)
         return self
 
     def predict(self, X=None):
@@ -69,7 +69,6 @@ class _BaseDMLATEIV(_OrthoLearner):
                  discrete_treatment=False,
                  categories='auto',
                  cv=2,
-                 n_splits='raise',
                  mc_iters=None,
                  mc_agg='mean',
                  random_state=None):
@@ -77,7 +76,6 @@ class _BaseDMLATEIV(_OrthoLearner):
                          discrete_instrument=discrete_instrument,
                          categories=categories,
                          cv=cv,
-                         n_splits=n_splits,
                          mc_iters=mc_iters,
                          mc_agg=mc_agg,
                          random_state=random_state)
@@ -221,7 +219,6 @@ class DMLATEIV(_BaseDMLATEIV):
                  discrete_instrument=False,
                  categories='auto',
                  cv=2,
-                 n_splits='raise',
                  mc_iters=None,
                  mc_agg='mean',
                  random_state=None):
@@ -232,7 +229,6 @@ class DMLATEIV(_BaseDMLATEIV):
                          discrete_treatment=discrete_treatment,
                          categories=categories,
                          cv=cv,
-                         n_splits=n_splits,
                          mc_iters=mc_iters,
                          mc_agg=mc_agg,
                          random_state=random_state)
@@ -298,7 +294,6 @@ class ProjectedDMLATEIV(_BaseDMLATEIV):
                  discrete_instrument=False,
                  categories='auto',
                  cv=2,
-                 n_splits='raise',
                  mc_iters=None,
                  mc_agg='mean',
                  random_state=None):
@@ -309,7 +304,6 @@ class ProjectedDMLATEIV(_BaseDMLATEIV):
                          discrete_treatment=discrete_treatment,
                          categories=categories,
                          cv=cv,
-                         n_splits=n_splits,
                          mc_iters=mc_iters,
                          mc_agg=mc_agg,
                          random_state=random_state)
@@ -387,7 +381,7 @@ class _BaseDMLIVModelFinal:
 
     def fit(self, Y, T, X=None, W=None, Z=None, nuisances=None, sample_weight=None, sample_var=None):
         Y_res, T_res = nuisances
-        self._model_final.fit(X, T_res, Y_res, sample_weight=sample_weight, sample_var=sample_var)
+        self._model_final.fit(X, T, T_res, Y_res, sample_weight=sample_weight, sample_var=sample_var)
         return self
 
     def predict(self, X=None):
@@ -485,14 +479,12 @@ class _BaseDMLIV(_OrthoLearner):
 
     def __init__(self, discrete_instrument=False, discrete_treatment=False, categories='auto',
                  cv=2,
-                 n_splits='raise',
                  mc_iters=None, mc_agg='mean',
                  random_state=None):
         super().__init__(discrete_treatment=discrete_treatment,
                          discrete_instrument=discrete_instrument,
                          categories=categories,
                          cv=cv,
-                         n_splits=n_splits,
                          mc_iters=mc_iters,
                          mc_agg=mc_agg,
                          random_state=random_state)
@@ -601,11 +593,12 @@ class _BaseDMLIV(_OrthoLearner):
 
         Returns
         -------
-        models_Y_X: list of objects of type(`model_Y_X`)
-            A list of instances of the `model_Y_X` object. Each element corresponds to a crossfitting
+        models_Y_X: nested list of objects of type(`model_Y_X`)
+            A nested list of instances of the `model_Y_X` object. Number of sublist equals to number of monte carlo
+            iterations, each element in the sublist corresponds to a crossfitting
             fold and is the model instance that was fitted for that training fold.
         """
-        return [mdl._model_Y_X._model for mdl in super().models_nuisance_]
+        return [[mdl._model_Y_X._model for mdl in mdls] for mdls in super().models_nuisance_]
 
     @property
     def models_T_X(self):
@@ -614,11 +607,12 @@ class _BaseDMLIV(_OrthoLearner):
 
         Returns
         -------
-        models_T_X: list of objects of type(`model_T_X`)
-            A list of instances of the `model_T_X` object. Each element corresponds to a crossfitting
+        models_T_X: nested list of objects of type(`model_T_X`)
+            A nested list of instances of the `model_T_X` object. Number of sublist equals to number of monte carlo
+            iterations, each element in the sublist corresponds to a crossfitting
             fold and is the model instance that was fitted for that training fold.
         """
-        return [mdl._model_T_X._model for mdl in super().models_nuisance_]
+        return [[mdl._model_T_X._model for mdl in mdls] for mdls in super().models_nuisance_]
 
     @property
     def models_T_XZ(self):
@@ -627,11 +621,12 @@ class _BaseDMLIV(_OrthoLearner):
 
         Returns
         -------
-        models_T_XZ: list of objects of type(`model_T_XZ`)
-            A list of instances of the `model_T_XZ` object. Each element corresponds to a crossfitting
+        models_T_XZ: nested list of objects of type(`model_T_XZ`)
+            A nested list of instances of the `model_T_XZ` object. Number of sublist equals to number of monte carlo
+            iterations, each element in the sublist corresponds to a crossfitting
             fold and is the model instance that was fitted for that training fold.
         """
-        return [mdl._model_T_XZ._model for mdl in super().models_nuisance_]
+        return [[mdl._model_T_XZ._model for mdl in mdls] for mdls in super().models_nuisance_]
 
     @property
     def nuisance_scores_Y_X(self):
@@ -676,10 +671,7 @@ class _BaseDMLIV(_OrthoLearner):
             feature_names = self._input_names["feature_names"]
         if self.original_featurizer is None:
             return feature_names
-        elif hasattr(self.original_featurizer, 'get_feature_names'):
-            return self.original_featurizer.get_feature_names(feature_names)
-        else:
-            raise AttributeError("Featurizer does not have a method: get_feature_names!")
+        return get_feature_names_or_default(self.original_featurizer, feature_names)
 
 
 class DMLIV(LinearModelFinalCateEstimatorMixin, _BaseDMLIV):
@@ -768,7 +760,6 @@ class DMLIV(LinearModelFinalCateEstimatorMixin, _BaseDMLIV):
                  featurizer=None,
                  fit_cate_intercept=True,
                  cv=2,
-                 n_splits='raise',
                  mc_iters=None,
                  mc_agg='mean',
                  discrete_instrument=False, discrete_treatment=False,
@@ -780,7 +771,6 @@ class DMLIV(LinearModelFinalCateEstimatorMixin, _BaseDMLIV):
         self.featurizer = clone(featurizer, safe=False)
         self.fit_cate_intercept = fit_cate_intercept
         super().__init__(cv=cv,
-                         n_splits=n_splits,
                          mc_iters=mc_iters,
                          mc_agg=mc_agg,
                          discrete_instrument=discrete_instrument,
@@ -898,7 +888,6 @@ class NonParamDMLIV(_BaseDMLIV):
                  model_final,
                  featurizer=None,
                  cv=2,
-                 n_splits='raise',
                  mc_iters=None,
                  mc_agg='mean',
                  discrete_instrument=False,
@@ -911,7 +900,6 @@ class NonParamDMLIV(_BaseDMLIV):
         self.model_final = clone(model_final, safe=False)
         self.featurizer = clone(featurizer, safe=False)
         super().__init__(cv=cv,
-                         n_splits=n_splits,
                          mc_iters=mc_iters,
                          mc_agg=mc_agg,
                          discrete_instrument=discrete_instrument,
